//******************************************************************************
//******************************************************************************
/**
 * @file    BPNGDefines.h
 * @author  Markus van Pinxteren
 * @date    12.05.2010
 *
 * @brief   Defines for Telemotive Client Library
*/
//******************************************************************************
//******************************************************************************
#pragma once
#ifndef __BPNGDEFINES_H__
#define __BPNGDEFINES_H__

#ifdef _MSC_VER

#ifdef BPNGCLIENTLIB_EXPORTS
#define DECLDIR __declspec(dllexport)
#else
#define DECLDIR __declspec(dllimport)
#endif
#include <windows.h>

#else   // not _MSC_VER

#include "cstdio"
#define WINAPI
#define DECLDIR
#define BOOL    bool
#define VOID    void

#endif  // _MSC_VER

#include "stdint.h"


#ifdef _WINDOWS
static const char* BPNG_CLIENT_LIB_VERSION = "2.4.2";
static const char* BPNG_STREAMING_LIB_VERSION = "1.3.7";
#else //not _WINDOWS
static const char BPNG_CLIENT_LIB_VERSION[] = "2.1.0.13";
static const char BPNG_STREAMING_LIB_VERSION[] = "1.1.2.1";
#endif //_WINDOWS

// ===========================================================================
/// enum Error codes
/**
  An error is identified by one of the following error codes. Additional
  information may be found in the BPNGError::msg field (e.g. file path that causes
  a BPNG_LOCAL_PATH_NOT_FOUND error)
*/
enum BPNGErrCode
{
    BPNG_NOERR = 0, ///< no error

    // connection errors
    BPNG_LOGGER_NOT_FOUND = 1,      ///< The IP address the lib wanted to connect was not found
    BPNG_NOT_CONNECTED = 2,         ///< A function call failed because the logger was not connected.
    BPNG_CONNECT_FTP_FAILED = 3,    ///< Establishing the ftp connection failed
    BPNG_CONNECT_TMPBUS_FAILED = 4, ///< Establishing the TMP (Telemotive Protocol) bus connection failed
    BPNG_TMPBUS_NOT_CONNECTED = 5,  ///< TMP bus is not connected
    BPNG_AMBIGUOUS_IP = 66,         ///< multiple devices with same IP available
    BPNG_FAILED_TO_CONNECT_STREAMING = 67, ///< Streaming feature could not be connected

    // FTP errors
    BPNG_FTP_NOT_CONNECTED = 6,            ///< FTP is not connected
    BPNG_FTP_SERVER_NOT_FOUND = 7,         ///< FTP server is not fould
    BPNG_FTP_LOGIN_FAILED = 8,             ///< FTP login failed
    BPNG_FTP_REMOTE_PATH_NOT_FOUND = 9,    ///< A requested path on the FTP server is not found
    BPNG_FTP_READ_REMOTE_FILE_ERROR = 10,   ///< Can't red a file on the FTP server
    BPNG_FTP_WRITE_REMOTE_FILE_ERROR = 11,  ///< Can't write a file on the FTP server
    BPNG_FTP_TRANSFER_USER_CANCELED = 12,   ///< FTP file transfer was canceled by the user
    BPNG_FTP_CREATE_REMOTE_DIR_ERROR = 13,  ///< Can't create the directory on the FTP server
    BPNG_FTP_REMOVE_REMOTE_DIR_ERROR = 14,  ///< Can't remove the directory on the FTP server
    BPNG_FTP_REMOVE_REMOTE_FILE_ERROR = 15, ///< Can't remove the file on the FTP server
    BPNG_FTP_CHANGE_CWD_ERROR = 16,         ///< Can't change the current working directory on the FTP server

    // TMPBus errors
    BPNG_TMPBUS_COPYRDB_ERROR = 17,  ///< Failed to copy the reference data base to the logger's tmp directory
    BPNG_TMPBUS_SEND_MSG_ERROR = 18, ///< Failed to send a TMP bus request message
    BPNG_TMPBUS_REQUEST_ERROR = 19,  ///< The TMP bus request execution failed

    // file handling error
    BPNG_FAILED_TO_CREATE_LOCAL_FILE_OR_DIRECTORY = 20, ///< Failed to create local file or directory
    BPNG_LOCAL_PATH_NOT_FOUND = 21,   ///< Local path not found
    BPNG_READ_LOCAL_FILE_ERROR = 22,  ///< Failed to read local file
    BPNG_WRITE_LOCAL_FILE_ERROR = 23, ///< Failed to write local file
    BPNG_FILE_EXISTS_ERROR = 24,      ///< Local file already exists
    BPNG_DIR_EXISTS_ERROR = 25,       ///< Local directory already exists
    BPNG_TARGET_PATH_TOO_LONG = 26,   ///< Specified path exceeds the max. valid length (e.g. 260 for Windows systems)
    BPNG_ZIP_EXCEEDS_FATFS_MAX = 27,  ///< ZIP file exceeds max size for FAT32 file systems

    // xml error
    BPNG_XML_PARSER_ERROR = 28, ///< Error while parsing xml file

    // user error
    BPNG_INITIALISATION_ERROR = 29, /**< BPNGClient instance is not initialised or with the wrong function.
                                         Use IBPNGClient::initOnline for data download or conversion directly from the device and
                                         IBPNGclient::iniOffline for data conversion from an offline data set.*/

    // SQLITE error
    BPNG_RDB_SQLITE_QUERY_ERROR = 30, ///< Error when trying to read data from the rdb.
    BPNG_RDB_OPEN_FAILED = 31,        ///< Failed to open the reference data base.

    // conversion error
    BPNG_CONVERSION_ERRORS = 32,        /**< Multiple conversion errors. Use IBPNGClient::getNumConversionErrors()
	                                    and IBPNGClient::getConversionError() for further information*/
    BPNG_CONV_SET_NOT_FOUND = 33,       ///< The passed conversion set pointer was not created with this IBPNGClient instance and dus could not be found.
    BPNG_NOTHING_TO_CONVERT = 34,       ///< There is no data available that could be converted. Check the specified time/id spans
    BPNG_TMT_FILE_ID_ERROR = 35,        ///< Invalid TMT/XTMT file id while trying to convert data.
    BPNG_TMT_FORMAT_ERROR_VERSION = 36, ///< The TMT/XTMT version of the trace file is not supported by this lib version.
    BPNG_TMT_FORMAT_ERROR_TS = 37,      ///< Missing FileTimeMessage in header of TMT/XTMT file
    BPNG_INVALID_MESSAGE_ERROR = 38,    ///< Invalid messages found in trace file(s).
    BPNG_INVALID_MESSAGE_ID = 39,       ///< Invalid message id found in trace file(s).
    BPNG_INVALID_MESSAGE_TS = 40,       ///< Invalid message ts found in trace file(s).
    BPNG_INVALID_MESSAGE_SUBID = 41,    ///< Invalid message sub id found in trace file(s).
    BPNG_INVALID_MESSAGE_LEN = 42,      ///< Invalid message length found in trace file(s).
    BPNG_CONV_FORMAT_ERROR = 43,        ///< Invalid format assignment or mismatching recorded trace data for the specified conversion format.

    // download error
    BPNG_DOWNLOAD_ERRORS = 44,     /**< Multiple download errors. Use IBPNGClient::getNumDownloadErrors()
	                               and IBPNGClient::getDownloadError() for further information*/
    BPNG_NOTHING_TO_DOWNLOAD = 45, ///< There is no data available that could be downoaded. Check the specified time/id spans

    // various
    BPNG_INVALID_OFFLINE_SET = 46,    ///< Failed to initialise the IBPNGClient from the passed offline data set
    BPNG_PARAMETER_MISMATCH = 47,     ///< currently not used
    BPNG_FW_VERSION_CHECK_ERROR = 48, ///< The verification of the new firmware at the end of a firmware update failed.
    BPNG_USER_CANCELLED = 49,         ///< currently not used
    BPNG_MIN_VERSION_ERROR = 50,      ///< The current library version does not suffice the the reuired min version written to BPNGError::msg
    BPNG_EXCEPTION = 51,              ///< Some kind of unhandled exception was thrown.
    BPNG_INCOMPATIBLE_RDB = 52,       ///< The logger's or offline data set's RDB-Verison is incompatible to this library version
    BPNG_UNSPECIFIED_ERROR = 53,      ///< An unspecified error occurred
    BPNG_LOAD_DBC_FAILED = 81, 

    // CCP XCP errors
    BPNG_CCP_XCP_PARSER_ERROR = 54,
    BPNG_CCP_XCP_DBC_GENERATOR_ERROR = 55,
    BPNG_CCP_XCP_SEQUENCE_GENERATOR_ERROR = 56,

    BPNG_INSUFFICIENT_DISK_SPACE = 57,
    BPNG_FWUPDATE_FAILED = 58,
    BPNG_INDEX_OUT_OF_RANGE_ERR = 59,
    BPNG_READ_CONFIG_BACKUP_ERR = 60,

    //TSL
    BPNG_INVALID_RPC_COMMAND = 61,          ///< if a rpc command for tsl is wrong
    BPNG_INVALID_TSL_CASCDING = 62,         ///< if cascading of tsl is invalid

    //PWd   
    BPNG_LOGIN_CANCELED = 63,
    BPNG_USER_PWD_WRONG = 64,
    BPNG_NO_ACCESS_FOR_FUNCTION = 65,
    
    // streaming error
    BPNG_STREAMING_PROTOCOLL_ERROR = 68,
    BPNG_STREAMING_SOCKET_ERROR = 69,
    BPNG_STREAMING_DISABLED = 70,
    BPNG_FW_DEPRECATED = 71,
    BPNG_STREAMING_ABORTED_BY_PEER = 72,

    // TSL
    BPNG_INCONSISTENT_TSL_FWVERSIONS = 80,  ///< if fw versions on tsl clusters are inconsistent
    BPNG_INVALID_TSL_CLUSTER = 82,          ///< in case of different TSLNetwork IDs

    // Plugins
    BPNG_DLL_NO_FORMAT_PLUGIN = 83,
    BPNG_FORMAT_PLUGIN_ID_EXISTS = 84,
    BPNG_DLL_NO_SYSTEMCLIENTLISTENER_PLUGIN = 90,

    // Resume downlaod error
    BPNG_FAILED_RENAME_RESUMED_OFFLINEDATASET = 85,
    BPNG_FAILED_RENAME_RESUMED_RDB = 86,
    BPNG_RESUME_INIT_FAILURE = 87,

    BPNG_SIGNAL_FILTER_INVALID_CONFIG = 88,
    BPNG_BAD_ALLOC = 89
    // Highest enum value is BPNG_DLL_NO_SYSTEMCLIENTLISTENER_PLUGIN = 90
};


enum FWUpdateErrorCode
{
    FWUPDATE_ERRORCODE_NO_ERR = 0,
    FWUPDATE_ERRORCODE_FW_PKT_NAME_EMPTY = -2,
    FWUPDATE_ERRORCODE_FW_PKT_MISSING = -3,
    FWUPDATE_ERRORCODE_NAMED_PIPE_SERVER_MKNOD = -4,
    FWUPDATE_ERRORCODE_NAMED_PIPE_SERVER_OPEN = -5,
    FWUPDATE_ERRORCODE_FW_UPDATE_NOT_IN_PROGRESS_TIMEOUT = -6,
    FWUPDATE_ERRORCODE_MISSING_LINUX_DISTR = -7,
    FWUPDATE_ERRORCODE_MISSING_LIBTMLIB_FILE = -8,
    FWUPDATE_ERRORCODE_MISSING_TMLIB_FILE = -9,
    FWUPDATE_ERRORCODE_MISSING_ATOM_FILE = -10,
    FWUPDATE_ERRORCODE_MISSING_CLIENT_FILE = -11,
    FWUPDATE_ERRORCODE_MISSING_FPGAA_FILE = -12,
    FWUPDATE_ERRORCODE_MISSING_FPGAB_FILE = -13,
    FWUPDATE_ERRORCODE_MISSING_EXTENSION_BOARD_FPGA_FILE = -14,
    FWUPDATE_ERRORCODE_MISSING_GBE_FILE = -15,
    FWUPDATE_ERRORCODE_MISSING_SBC_FILE = -16,
    FWUPDATE_ERRORCODE_MISSING_SBC_FLASH_SCRIPT = -17,
    FWUPDATE_ERRORCODE_MISSING_FPGA_FLASH_SCRIPT = -18,
    FWUPDATE_ERRORCODE_MISSING_RCV_FILE = -19,
    FWUPDATE_ERRORCODE_MISSING_LINUX_SETUP_ARCHIVE = -20,
    FWUPDATE_ERRORCODE_UNKNOWN_MB_HW_VERSION = -21,
    FWUPDATE_ERRORCODE_UNKNOWN_EXTENSION_BOARD = -22,
    FWUPDATE_ERRORCODE_UNKNOWN_EXTENSION_BOARD_VARIANCE = -23,
    FWUPDATE_ERRORCODE_NOT_READABLE_EXTENSION_BOARD_VARIANCE = -24,
    FWUPDATE_ERRORCODE_NOT_READABLE_EXTENSION_BOARD_HW_VERSION = -25,
    FWUPDATE_ERRORCODE_NOT_READABLE_HW_TYPE_VERSION = -26,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_APP_LIBS = -27,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_RC = -28,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_GBEC = -29,
    FWUPDATE_ERRORCODE_CONV_CFG_ERROR = -30,
    FWUPDATE_ERRORCODE_FAILED_UNCOMPRESS_LINUX_KERNEL = -31,
    FWUPDATE_ERRORCODE_FAILED_UNCOMPRESS_LINUX_KERNEL_MODULES = -32,
    FWUPDATE_ERRORCODE_FAILED_CPY_LINUX_KERNEL = -33,
    FWUPDATE_ERRORCODE_FAILED_UNCOMPRESS_CLIENT_FILE = -34,
    FWUPDATE_ERRORCODE_FAILED_CPY_CLIENT_FILE = -35,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_LINUX_DISTR = -36,
    FWUPDATE_ERRORCODE_FAILED_SBC_FLASH = -37,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_CCP_XCP = -38,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_CCP_XCP_SEED_KEY_SERVERS = -39,
    FWUPDATE_ERRORCODE_MISSING_CCP_XCP_FILE = -40,
    FWUPDATE_ERRORCODE_MISSING_CCP_XCP_SEED_KEY_SERVER_FILE = -41,
    FWUPDATE_ERRORCODE_MISSING_SPYNIC_FILE = -42,
    FWUPDATE_ERRORCODE_MISSING_LOADING_ISPVM = -43,
    FWUPDATE_ERRORCODE_MISSING_DEVICE_FPGAB_FILE = -44,
    FWUPDATE_ERRORCODE_MISSING_DEVICE_FPGAA_FILE = -45,
    FWUPDATE_ERRORCODE_UNREADY_FPGAA = -46,
    FWUPDATE_ERRORCODE_LINUX_KERNEL_MAY_FREEZE_SYSTEM = -47,
    FWUPDATE_ERRORCODE_NOT_SET_DEVICE_PATH = -48,
    FWUPDATE_ERRORCODE_NOT_SET_FW_FILE = -49,
    FWUPDATE_ERRORCODE_NOT_SET_FPGA_KEY = -50,
    FWUPDATE_ERRORCODE_MISSING_DEVICE_FILE = -51,
    FWUPDATE_ERRORCODE_MISSING_FPGA_FW_FILE = -52,
    FWUPDATE_ERRORCODE_MISSING_TMUDEVQ = -53,
    FWUPDATE_ERRORCODE_UNKNOWN_DEVICE_PATH = -54,
    FWUPDATE_ERRORCODE_FAILED_CPY_FPGA_FILE = -55,
    FWUPDATE_ERRORCODE_FAILED_LINKING_FW_UPDATE = -56,
    FWUPDATE_ERRORCODE_ERROR_FLASH_FPGA = -57,
    FWUPDATE_ERRORCODE_ERROR_FLASH_SPYNIC = -58,
    FWUPDATE_ERRORCODE_ERROR_LOADING_FPGA_JTAG_DRIVER = -59,
    FWUPDATE_ERRORCODE_MISSING_EXTENSION_BOARD_VIA_PCIE = -60,
    FWUPDATE_ERRORCODE_FAILED_CONV_CFG = -61,
    FWUPDATE_ERRORCODE_FAILED_UPLOAD = -62,
    FWUPDATE_ERRORCODE_FWUFOLDER_EXISTS = 63,
    FWUPDATE_ERRORCODE_UNDEFINED = -1
};

/// Warning codes
/**
  Warnings are notified by listener calls to the function IBPNGClientListener::onWarning()
*/
enum BPNGWarningCode
{
    BPNG_NOWARNING, /// no warning
    BPNG_WARNING_CLOSE_TRACE_FILES, ///< Failed to close the current trace files on the logger device when trying to execute IBPNGClient::initOnline()
    BPNG_WARNING_MESSAGES_NOT_CONVERTED, ///< In case of protocol mismatch between recorded data and target format or unsupported message sub types, it is possible that some messages can not be converted to the selected format.
    BPNG_WARNING_NO_ESO_TRACE, ///< ethernet data for eso trace conversion is not logged in eso trace format
    BPNG_WARNING_TSL_WITH_DIFFERENT_TIMEZONES, ///< A TSL cluster with loggers with different time zones is in undefined state. It's not defined which time zone will be used for time zone dependent processes.
    BPNG_WARNING_RECOVERING_FAILED ///< Recovering trace files from a previous power down failed
};

/// Languages
/**
  ID for specifiying the language in that the library handles process and error information.
  Default language is english.
*/
enum LanguageID
{
    BPNG_GERMAN, ///< english
    BPNG_ENGLISH ///< german
};

// ===========================================================================
/// Mode for the IBPNG::downloadBugReport() function
enum BPNGBugreportMode
{
    BR_FULL_WO_TRACES = 0,      ///< Full bug report without traces
    BR_ONLY_LOGS = 1,           ///< Only log files are downloaded
    BR_FDB_RDB = 2,             ///< only FDB and RDB are downloaded
    BR_ONLY_CLIENT = 3,         ///< only client logs are stored
    BR_FULL_ALL_TRACES = 4,     ///< Full bug report with all traces files
    BR_FULL_TIMESPAN_TRACES = 5 ///< Full bugreport with trace file of a specified time span
};

// ===========================================================================
/// Currently supported interfaces
enum ChannelType
{
    CH_UNDEFINED = 0, ///< undefined channel type
    OBSOLETE_CH_CANLS,///< CAN low speed interface
    CH_CAN,           ///< CAN high speed interface
    CH_LIN,           ///< LIN interface
    CH_SERIAL,        ///< Serial interface
    CH_ETHERNET,      ///< Ethernet interface
    CH_FLEXRAY,       ///< Flexray interface
    CH_MOST25_CTRL,   ///< MOST 25 control channel
    CH_MOST25_MDP,    ///< MOST 25 data packet channel (MDP)
    CH_MOST25_SYNC,   ///< MOST 25 synchroneous channel (streaming data)
    CH_MOST150_CTRL,  ///< MOST 150 control channel
    CH_MOST150_MDP,   ///< MOST 150 data packet channel (MDP)
    CH_MOST150_MEP,   ///< MOST 150 ethernet packet channel (MEP)
    CH_MOST150_STREAM,   ///< MOST 150 synchroneous channel (streaming data)
    CH_ANALOG_IN,     ///< Analog in
    CH_DIGITAL_IN,    ///< Digital in
    CH_CAMERA,        ///< Camera channel
    CH_CCPXCP,        ///< CCP XCP
    CH_DIAG,          ///< Diagnose, currently not used
    CH_GPS,           ///< Global Positioning System
    CH_ECL,           ///< Electronic Control Line
    CH_COMPLEXFILTER, ///< 
    CH_TTY            ///< TTY channel for QXDM
};


// ===========================================================================
/// Identifier for currently supported formats
static const char* UNDEFINED = "UNDEFINED";
static const char* TMASC = "TMASC";	                           ///< Telemotive ASCII format
static const char* OP2 = "OP2";		                           ///< MOST OP" format
static const char* CANOE = "CANOE";	                           ///< CANOE ASCII format
static const char* STA = "STA";		                           ///< Serial Trace Analyser format
static const char* GNLOG = "GNLOG";	                           ///< GNLog format
static const char* TCLOG = "TCLOG";	                           ///< Trace Client format
static const char* TCLOG_TS = "TCLOG_TS";                          ///< Trace Client format plus time stamps
static const char* RAW_SERIAL = "RAW_SERIAL";                      ///< Raw Serial Format
static const char* IMG = "IMG";		                           ///< DataAnalyser IMG format
static const char* TMBIN = "TMBIN";	                           ///< Telemotive binary format
static const char* CANCORDER = "CANCORDER";                        ///< CANCorder format
static const char* APN = "APN";		                           ///< APN serial format
static const char* ASCHEX = "ASCHEX";	                           ///< ASCII format that writes binary data as hex values
static const char* TCPDUMP = "TCPDUMP";	                           ///< TCP dump (*.pcap) format
static const char* BLF = "BLF";		                           ///< Binary Logging Format - Vector Informatik
static const char* DLT_BMW = "DLT_BMW";	                           ///< BMW specific DLT Format
static const char* MDF = "MDF";		                           ///< Measurement data format - Vector Informatik
static const char* MDF_CAN_SIG = "MDF_CAN_SIG";                    ///< MDF format based on CAN signals
static const char* MPEG4_BLOCKS = "MPEG4_BLOCKS";                  ///< Camera format seperated blocks
static const char* MPEG4_JOINED = "MPEG4_JOINED";                  ///< Camera joined videos
static const char* NMEA = "NMEA";	                           ///< Raw format for GPS data
static const char* KML = "KML";		                           ///< KML format for GPS data
static const char* KMZ = "KMZ";		                           ///< KMZ format for GPS data
static const char* SERIAL_DEBUG = "SERIAL_DEBUG";                  ///< Serial Format with "new Line"-packaging
static const char* RAW_ETHERNET = "RAW_ETHERNET";                  ///< Raw ethernet format
static const char* ESO_TRACE = "ESO_TRACE";                        ///< Audi specific ESO Format
static const char* XTMT = "XTMT";	                           ///< Extended Telemotive Trace File format
static const char* GPX = "GPX";		                           ///< GPS Exchange format
static const char* MDF_CCP_XCP_SIG = "MDF_CCP_XCP_SIG";            ///< MDF format based on CCP & XCP signals.
static const char* TS = "TS";                                      ///< Raw isochrone format
static const char* FILTER_FORMAT = "FILTER_FORMAT";                ///< Filter format= no change in format
static const char* MDF_CAN_SIG_4_10 = "MDF_CAN_SIG_4_10";          ///< MDF 4.10 format based on CAN signals
static const char* MDF_CCP_XCP_SIG_4_10 = "MDF_CCP_XCP_SIG_4_10";  ///< MDF 4.10 format based on CCP & XCP signals.
static const char* INVALID = "INVALID";




// ===========================================================================
///
enum PwdPrivilegesFuncId
{
    REMOVE_DATA = 0,
    SET_TIME,
    SET_EVENT,
    RECONFIG,
    RECONFIG_PASSWORD,
    RECONFIG_COMPLEX_FILTER,
    UPLOAD_WINE_DLLS,
    UPDATE_FIRMWARE,
    CHANGE_LICENCES,

    // must be last enum value
    PRIVILEGES_END
};


// ===========================================================================
// List of possible reasons why a measure signal was ignored
enum Reason
{
    R_UNSUPPORTED_BIT_MASK,     ///< DBC file don't support bit operations with a bit mask.
    R_BIT_MASK_OVERLAP,         ///< Bit mask is incorrect and cause a overlap with at least one other signal.
    R_UNSUPPORTED_COMPU_TAB,    ///< DBC file don't support all compu tab types; only tab will ignored, not the signal itself!
    R_FORBIDDEN_TAB_VALUE,      ///< DBC file don't support all possible values of a compu tab; only tab will ignored, not the signal itself!
    R_UNKNOWN                   ///< Unknown reason.
};

// ===========================================================================
/// False measure signal interface
struct IFalseMeasureSignal
{
    /// Returns the device Id
    virtual uint8_t getDeviceId() const = 0;
    /// Returns the signal number
    virtual uint16_t getSignalNo() const = 0;
    /// Returns the ignore reason
    virtual Reason getIgnoreReason() const = 0;
};

// ===========================================================================
/// False measure signal list interface
struct IFalseMeasureSignalList
{
    /// Returns the number of signals
    virtual size_t getSize() const = 0 ;
    /// Returns the IFalseMeasureSignal at index
    virtual const IFalseMeasureSignal* getSignal(size_t index) const = 0;
};

// ===========================================================================
/// Channel interface
struct IChannel
{
    /// Returns the ChannelType
    virtual ChannelType getType() const = 0;
    /// Returns the channel's index
    virtual uint8_t getIndex() const = 0;
    /// Returns the channel's name
    virtual const char* getName() const = 0;
    /// Returns the channel's package id
    virtual uint32_t getMainboardNumber() const = 0;
    /// Returns the channel's offset
    virtual uint32_t getOffset() const = 0;
    /// Returns whether the channel is mapped
    virtual BOOL isMappingActive() const = 0;
    /// Returns the channel's mapped channel index
    virtual uint8_t getMappedChannelIndex() const = 0;
};

// ===========================================================================
/// Channel interface
struct ITesttoolsChannel
{
    /// Returns the IChannel of this ITesttoolsChannel
    virtual IChannel* getIChannel() const = 0;
    /// Returns whether the channel matches with the contained channel
    virtual BOOL matchIChannel(const IChannel* iChannel) const = 0;
    /// Returns the channel's containerId
    virtual uint32_t getContainerId() const = 0;
    /// Returns the channel's associated containerId
    virtual uint32_t getPseudoContainerId() const = 0;
    /// Returns the channel's associated containerId name
    virtual const char* getPseudoChannelName() const = 0;
    /// Returns the channel's containerId
    virtual uint16_t getBaseCanId() const = 0;
    /// Returns the channel's containerId is extended or not
    virtual bool isExtendedCanId() const = 0;
    /// Returns the ethernet host ip
    virtual const char* getHostIp() const = 0;
    /// Returns the ethernet device ip
    virtual const char* getDeviceIp() const = 0;
    /// Returns the ethernet device ip
    virtual unsigned int getDevicePort() const = 0;
    /// Returns protocol
    virtual int getProtocol() const = 0;
    /// Returns debuglevel
    virtual int getDebugLevel() const = 0;
};

// ===========================================================================
/// Channel list interface
struct IChannelList
{
    /// Returns the number of channels
    virtual int getSize() const = 0 ;
    /// Returns the IChannel at index
    virtual const IChannel* getChannel(int index) const = 0;
};

// ===========================================================================
/// TesttoolsChannel list interface
struct ITesttoolsChannelList
{
    /// Returns the number of channels
    virtual int getSize() const = 0 ;
    /// Returns the ITesttoolsChannel at index
    virtual const ITesttoolsChannel* getTesttoolsChannel(int index) const = 0;
};


// ===========================================================================
/// FormatInfo interface
struct IFormatInfo
{
    /// Returns the FormatId
    virtual const char* getFormatId() const = 0;

    /// Returns the format's description name in the language passed as ISO 639-1 language code ("en", "de", etc.)
    virtual const char* getName(const char* language) const = 0;

    /// Returns whether the format supports multiple channels in one output file
    virtual BOOL isMultipleChannelSupport() const = 0;

    /// Returns whether the format is binary
    // 0 = false, else = true
    virtual BOOL isBinaryFormat() const = 0;

    /// Returns the format's default extension
    virtual const char* getExtension() const = 0;

    /// Returns the number of supported channel types
    virtual int getNumSupportedChannelTypes() const = 0;

    /// Returns one supported ChannelType
    virtual ChannelType getChannelType(int index) const = 0;

    /// Returns the required license for the format, an empty string for free formats
    virtual const char* getRequiredLicense() const = 0;
};

// ===========================================================================
/// Format list interface
struct IFormatList
{
    /// Returns the number of available formats
    virtual int getSize() const = 0;

    /// Returns the IFormat at index
    virtual const IFormatInfo* getFormatInfo(int index) const = 0;
};

// ===========================================================================
// Conversion set interface
/// A conversion set stores all conversion relevant settings
/**
  To convert trace data a conversion set must be created.
  Several channels can be added to one conversion set. The trace data
  of that channels are converted to the assigned formats.
  The conversion set also includes the data spans that has to be converted.
*/
struct IConversionSet
{
    /// Adds a channel to the conversion set and assigns the target format to it.
    /**
      Use the IBPNGClient::getLoggerChannel() function to get all existing channels.

      Hint for offset, mappingActive and mappedChannelId:  Use the configured values! Else the channel will not be found and the data not written. All information can be retrieved from \a IChannel

      @param channelType must be one of the appropriate ChannelType enum.
      @param channelIndex zero-based channel index
      @param formatId must be one of the appropriate FormatId enum.
      @param fileId The data of all channels with same formatId and same fileId are written to the same output file. The default value -1 indicates always a separate file for each channel.
      @param offset Only needed for TSL, default 0. The offset to the original channel number. Can be read out from \a IChannel
      @param mbnr Only needed for TSL, default -1. The mainboardnumber of the channels source device. Can be read out from \a IChannel
      @param mappingActive Only needed for Channelmapping, default false. If true the mappedChannelId will be used instead of original index. Can be read out from \a IChannel
      @param mappedChannelId Only needed for Channelmapping, default -1. If mappingActive is true the mappedChannelId will be used instead of original index. Can be read out from \a IChannel
    */
    virtual void addChannel(ChannelType channelType, uint8_t channelIndex, const char* formatId, int fileId = -1, int offset = 0, int mbnr = -1, bool mappingActive = false, int mappedChannelId = -1) = 0;

    /// Adds a time span to the conversion set.
    /**
      The data within the time span will be converted to the specified formats.
      @param startTime must be in usec since 01.01.1970 (UTC)
      @param endTime must be in usec since 01.01.1970 (UTC)
    */
    virtual void addTimeSpan(uint64_t startTime, uint64_t endTime) = 0;

    /// Adds a ReferenceDB ID range to the conversion set.
    /**
       Passed parameter are IDs from the Reference Data Base (RDB). After calling
       on of the init functions IBPNGClient::initOnline() or IBPNGClient::initOffline()
       you can get the path to the RDB with IBPNGClient::getReferenceDataBasePath().

       The RDB includes all occurred events like startups, shutdowns, etc. but also
       all recorded trace files. Each RDB entry has a unique DataBaseEntryID.
       With this function you can easily select data between arbitrary RDB entries.
       For example you can convert all data between index X (which is e.g. a startup)
       and index Y (which is e.g. a shutdown). When the DataBaseEntryId of a trace
       file is passed, this trace block will be included by the conversion.

       @param startId DataBaseEntryId that inicates the start of the data range to be converted
       @param endId DataBaseEntryId that inicates the end of the data range to be converted
    */
    virtual void addRdbIdRange(uint64_t startId, uint64_t endId) = 0;

    /// Loads the format settings from an ini file
    virtual bool loadFormats(const char* pathToIniFile) = 0;

    /// Saves the format settings to an ini file
    virtual bool saveFormats(const char* pathToIniFile) = 0;
};

/// Logger status
enum BPNGLoggerStatus
{
    LS_OK = 0,          ///< Device is ok
    LS_ERROR = 1,       ///< Device has at least one active error
    LS_NOSYNC = 2,      ///< Device is configured as slave but no master is found
    LS_WARNING = 3,     ///< Device hast at least one active warning
    LS_FWUPDATE = 4,    ///< Firmeware update in progress
    LS_MEM = 5,         ///< Internal storage of device is full. Ring buffer deactivated or full with protected trace files.
    LS_RING = 6,        ///< Internal storage of device is full. Ring buffer is activated.
    LS_UNDEFINED = -1
};

// ===========================================================================
/// Enumartion of Telemotives next generation data loggers
enum BPNGDeviceType
{
    DEV_BP2,                ///< \deprecated For blue PiraT 2 devices use type \a DEV_BP2_V1X, for new blue PiraT 2 5E devices use \a DEV_BP2_V2X
    DEV_BPMINI,             ///< blue PiraT mini devices
    DEV_BP2_V1X,            ///< standard blue PiraT 2 device
    DEV_BP2_V2X,            ///< blue PiraT 2 5E device
    DEV_RC_TOUCH,           ///< Remote Control Touch
    DEV_BP_REMOTE,          ///< blue PiraT Remote

    DEV_TSL = 0x80,         ///< internal use only! don't use!
    DEV_UNKNOWN = 0xFF
};

// ===========================================================================
/// Struct with information about a logger found in LAN
struct OnlineLoggerInfo
{
    const char* ip;             ///< the logger's ip address
    const char* name;           ///< the logger's name
    const char* mbnr;           ///< mainboard number
    const char* deviceSN;       ///< device serial number, since FW 2.2.1
    uint8_t occupied;           ///< 0 = not occupied, 1 = connected with client, 2 = occupied by temp config (via external media)
    const char* currentUser;    ///< user name of connected pc account
    uint8_t loggerStatus;       ///< current logger status, @sa BPNGLoggerStatus
    uint8_t wlan;               ///< Flag for connection type. 0 = ethernet, 1 = wlan
    const char* tslEth0IP;      ///< ip address of device connected to eth0, 0.0.0.0 if none
    const char* tslEth1IP;      ///< ip address of device connected to eth1, 0.0.0.0 if none
    int8_t tslId;               ///< id for device in tsl network, continues in tsl, starts with 0 on first device
    int32_t tslNetworkId;       ///< id of tsl network, -1 = no TSL, all devices with same tslNetworkId belong to the same TSL
    const char* tslName;        ///< name(id) of tsl network
    uint8_t deviceType;         ///< Device type, @sa BPNGDeviceType
    const char* fwVersion;      ///< Current firmware version, since FW 2.1.1
};

// ===========================================================================
/// Types for DataSpan
enum DataSpanType
{
    DST_IDSPAN = 0,
    DST_TIMESPAN = 1
};

// ===========================================================================
///
struct DataSpan
{
    uint8_t type;   ///< set type to 0 for a id based range, set type to 1 for a time based range
    uint64_t start; ///< start time/id of data span
    uint64_t end;   ///<  end time/id of data span
};

// ===========================================================================
/// Error struct with error code and optional error message
struct BPNGError
{
    BPNGErrCode code; ///< error code
    const char* msg;  ///< error message
};


// ===========================================================================
/// structure for login
struct LogInData
{
    const char* userName;
    const char* userPwd;
};

// ===========================================================================
/// Common properties, \deprecated use \a IClientProperties interface instead
struct CommonProperties
{
    char* nameOfTester;
    uint32_t maxOutputSizeMB;
    uint8_t separatedTimeFormat;

    uint8_t separatedTimeFormatInOfflineSet;
    uint32_t maxOutputSizeMBSortedDownload;

    char* alternativeLoggerName;
    uint8_t useAlternativeLoggerName;
    uint8_t useSubdirectories;
    uint8_t midnightSplitting;
    uint8_t fileTimeSpansLikeSelection;

    uint8_t markerNumbersInFileNames;
    uint8_t subfolderWithLoggerName;

    uint32_t reserved1;
    uint32_t reserved2;
    uint32_t reserved3;
    uint32_t reserved4;
};

/// CAN pseudo message properties, \deprecated use \a IClientProperties interface instead
struct CANPseudoMessagesProperties
{
    uint8_t canTimeStampMessage;
    uint32_t channelTS;
    uint32_t dlcTS;
    uint32_t canIdTS;
    uint32_t hourBitPos;
    uint32_t minuteBitPos;
    uint32_t secondBitPos;
    uint32_t dayBitPos;
    uint32_t monthBitPos;
    uint32_t yearBitPos;
    uint8_t canTriggerMessage;
    uint32_t channelTrigger;
    uint32_t dlcTrigger;
    uint32_t canIdTrigger;
    uint32_t triggerNumBitPos;
    uint32_t reserved1;
    uint32_t reserved2;
    uint32_t reserved3;
    uint32_t reserved4;
};

/// MOST pseudo properties, \deprecated use \a IClientProperties interface instead
struct MOSTPseudoMessagesProperties
{
    uint8_t writeMessages;  ///< Active flag for writing MOST pseudo messages for trigger
    uint32_t source;          ///< Source address
    uint32_t target;          ///< Target address
    uint32_t functionBlockID; ///< Function block ID
    uint32_t functionID;      ///< Function ID
    uint32_t reserved1;
    uint32_t reserved2;
    uint32_t reserved3;
    uint32_t reserved4;
};

/// Conversion Properties, \deprecated use \a IClientProperties interface instead
struct ConversionProperties
{
    uint8_t useSatelliteTimeForGPSFormats;
    char* isochronalMost150Channels;
};

/// ClientProperties, \deprecated use \a IClientProperties interface instead
struct ClientProperties
{
    CommonProperties common;
    CANPseudoMessagesProperties canPseudoMessages;
    MOSTPseudoMessagesProperties mostPseudoMessages;
    ConversionProperties conversionProp;
};

/// stores memory fill level of a device
struct MemoryFillLevel
{
    uint16_t ringBufferSize;            ///< size of ringbuffer in GBytes
    uint8_t percentageFill;             ///< percentage filled
    uint8_t percentageFillProtected;    ///< percentage filled of protected areas
    uint64_t mbnr;                      ///< mainboardnumber of device
};

/// Pointer to a function named onLogRequest with one parameter and no return value
typedef void (WINAPI*onLogRequest)(const char* logRecord);

#endif // __BPNGDEFINES_H__
